import { Component } from "./component";
import { DrawLayerManager } from "./draw_layer_manager";
import { Type } from "./types/generic_type";

export abstract class DrawComponent extends Component {
    private _layer: number;
    private draw_layer_manager: DrawLayerManager<DrawComponent>;

    get layer(): number { return this._layer; }
    set layer(num: number) {
        if (num === this._layer) { return; }
        this.draw_layer_manager.adjust_object_layer(this, this._layer, num);
        this._layer = num;
    }

    constructor(
        DrawComponentType: Type<DrawComponent>,
        entity_id: number,
        layer_manager: DrawLayerManager<DrawComponent>,
    ) {
        super(DrawComponentType, entity_id);
        this._layer = 0;
        this.draw_layer_manager = layer_manager;
        this.draw_layer_manager.register_object_with_layer(this, this._layer);
    }

    public activate() {
        this.draw_layer_manager.register_object_with_layer(this, this._layer);
        super.activate();
    }

    public deactivate() {
        this.draw_layer_manager.unregister_object_with_layer(this, this._layer);
        super.deactivate();
    }

    public destroy() {
        this.draw_layer_manager.unregister_object_with_layer(this, this._layer);
        super.destroy();
    }
}
