import { GCOptimizedMap, GCOptimizedSet, GCOptimizedSortedList } from "tstl-gc-optimized-collections";

export class DrawLayerManager<T> {
    private _draw_order: T[] = [];

    private layer_index_to_component_set: GCOptimizedMap<number, GCOptimizedSet<T>>;
    private order: GCOptimizedSortedList<number>;

    get draw_order(): T[] { return this._draw_order; }

    constructor(
        layer_index_to_component_set: GCOptimizedMap<number, GCOptimizedSet<T>>,
        order: GCOptimizedSortedList<number>,
    ) {
        this.layer_index_to_component_set = layer_index_to_component_set;
        this.order = order;
    }

    public register_object_with_layer(object: T, layer: number) {
        this.insert(object, layer);
    }

    public unregister_object_with_layer(object: T, layer: number) {
        if (this.layer_index_to_component_set.has(layer)) {
            this.layer_index_to_component_set.get(layer)!.delete(object);
        }
    }

    public adjust_object_layer(object: T, old_layer: number, new_layer: number) {
        this.unregister_object_with_layer(object, old_layer);
        this.register_object_with_layer(object, new_layer);
    }

    public insert(draw_component: T, layer: number) {
        if (this.layer_index_to_component_set.has(layer)) {
            const set = this.layer_index_to_component_set.get(layer)!;
            set.add(draw_component);
        } else {
            const set = new GCOptimizedSet<T>();
            this.layer_index_to_component_set.set(layer, set);
            set.add(draw_component);
            this.order.add(layer);
        }
    }
}
