import { GCOptimizedSet } from "tstl-gc-optimized-collections";

export class EntityTracker {
    public tracked_ids = new GCOptimizedSet<number>();
    public deactivated_ids = new GCOptimizedSet<number>();

    public track_entity(id: number) {
        if (this.tracked_ids.has(id)) { return; }
        this.tracked_ids.add(id);
    }

    public untrack_entity(id: number) {
        if (!this.tracked_ids.has(id)) { return; }
        this.tracked_ids.delete(id);
        this.deactivated_ids.delete(id);
    }

    public on_entity_activated(id: number) {
        this.deactivated_ids.add(id);
        this.tracked_ids.delete(id);
    }

    public on_entity_deactivated(id: number) {
        this.deactivated_ids.delete(id);
        this.tracked_ids.add(id);
    }
}
