import { GCOptimizedList, GCOptimizedMap } from "tstl-gc-optimized-collections";
import { ComponentManager } from "./component_manager";
import { DrawLayerManager } from "./draw_layer_manager";
import { Entity } from "./entity";
import { IDrawable } from "./interfaces/drawable";

export class EntityCreator {
    private entities: GCOptimizedList<Entity>;
    private entities_with_added_components: GCOptimizedList<Entity>;
    private entities_with_removed_components: GCOptimizedList<Entity>;
    private entities_marked_for_destroy: GCOptimizedList<Entity>;

    private next_id_value: number = 1;
    private id_to_entity: GCOptimizedMap<number, Entity>;

    private component_manager: ComponentManager;
    private draw_layer_manager: DrawLayerManager<IDrawable>;

    constructor(
        entities: GCOptimizedList<Entity>,
        id_to_entity: GCOptimizedMap<number, Entity>,
        entities_with_added_components: GCOptimizedList<Entity>,
        entities_with_removed_components: GCOptimizedList<Entity>,
        entities_marked_for_destroy: GCOptimizedList<Entity>,
        component_manager: ComponentManager,
        draw_layer_manager: DrawLayerManager<IDrawable>,
    ) {
        this.entities = entities;
        this.id_to_entity = id_to_entity;
        this.entities_with_added_components = entities_with_added_components;
        this.entities_with_removed_components = entities_with_removed_components;
        this.entities_marked_for_destroy = entities_marked_for_destroy;
        this.component_manager = component_manager;
        this.draw_layer_manager = draw_layer_manager;
    }

    public create_entity(): Entity {
        const id = this.next_id();
        const entity = new Entity(
            id,
            this.entities_with_added_components,
            this.entities_with_removed_components,
            this.entities_marked_for_destroy,
            this.draw_layer_manager,
            this.component_manager,
        );

        this.entities.add(entity);
        this.id_to_entity.set(id, entity);

        return entity;
    }

    private next_id() {
        const value = this.next_id_value;
        this.next_id_value += 1;
        return value;
    }
}
