export abstract class Graph<T> {
    protected _vertices: T[] = [];
    protected _edges = new Map<T, T[]>();

    get vertices() {
        return this._vertices;
    }

    public neighbors(vertex: T): T[] {
        if (this.vertex_exists(vertex)) {
            return [...this._edges.get(vertex)!];
        } else {
            return [];
        }
    }

    public add_vertex(vertex: T) {
        if (!this.vertex_exists(vertex)) {
            this._vertices.push(vertex);
            this._edges.set(vertex, []);
        }
    }

    public add_vertices(...vertices: T[]) {
        for (const vertex of vertices) {
            this.add_vertex(vertex);
        }
    }

    public vertex_exists(vertex: T): boolean {
        return this.vertices.indexOf(vertex) !== -1;
    }

    public remove_vertex(v: T) {
        if (this.vertex_exists(v)) {
            for (const u of this.neighbors(v)) {
                this.remove_edge([v, u]);
            }

            this.vertices.splice(this.vertices.indexOf(v)!, 1);
        }
    }

    public abstract remove_edge(edge: [T, T]): void;
}
