import { Type } from "./types/generic_type";

/**
 * Messages are collections of data which are consumed
 * by Processors.
 *
 * Messages are used to send data to Engines so they can
 * modify the World state.
 *
 * Unlike Components, Messages are temporary and destroyed
 * at the end of each World update.
 *
 * For performance reasons, it is discouraged to create
 * new objects to pass to messages, as this will cause garbage
 * collection issues.
 *
 * Defining any logic on a Message is an anti-pattern.
 * Messages should not need to directly trigger any side-effects.
 */
export abstract class Message {
    public readonly type: Type<Message>;

    constructor(type: Type<Message>) {
        this.type = type;
    }
}
