import { GCOptimizedSet } from "tstl-gc-optimized-collections";
import { Message } from "./message";
import { EntityMessage } from "./types/entity_message";

export class MessageQuery {
    public message_query_set = new GCOptimizedSet<Message>();

    public init(message_set: GCOptimizedSet<Message>): MessageQuery {
        this.message_query_set.clear();
        for (const message of message_set.entries()) {
            this.message_query_set.add(message);
        }
        return this;
    }

    public filter_by_entity_id(entity_id: number): MessageQuery {
        for (const message of this.message_query_set.entries()) {
            if (this.isEntityMessage(message)) {
                if (message.entity.id !== entity_id ) {
                    this.message_query_set.delete(message);
                }
            } else {
                this.message_query_set.delete(message);
            }
        }
        return this;
    }

    public finish(): GCOptimizedSet<Message> {
        return this.message_query_set;
    }

    private isEntityMessage(message: Message): message is EntityMessage {
        return (message as EntityMessage).entity !== undefined;
    }
}
