import { GCOptimizedMap, GCOptimizedSet, GCOptimizedSortedList } from "tstl-gc-optimized-collections";
import { DrawComponent } from "./draw_component";
import { DrawLayerManager } from "./draw_layer_manager";
import { Entity } from "./entity";
import { IDrawable } from "./interfaces/drawable";
import { EntityRenderer } from "./renderers/entity";
import { TypeGuard } from "./type_guards";
import { Type } from "./types/generic_type";

export class RenderManager {
    public draw_layer_manager: DrawLayerManager<IDrawable>;
    private draw_component_type_to_renderer = new GCOptimizedMap<Type<DrawComponent>, EntityRenderer>();
    private entity_to_id_map: GCOptimizedMap<number, Entity>;

    private layer_index_to_drawable_set = new GCOptimizedMap<number, GCOptimizedSet<IDrawable>>();
    private order = new GCOptimizedSortedList<number>();

    constructor(entity_to_id_map: GCOptimizedMap<number, Entity>) {
        this.entity_to_id_map = entity_to_id_map;
        this.draw_layer_manager = new DrawLayerManager<IDrawable>(this.layer_index_to_drawable_set, this.order);
    }

    public register_entity_renderer(renderer: EntityRenderer) {
        this.draw_component_type_to_renderer.set(renderer.draw_component_type, renderer);
    }

    public draw() {
        for (const [_, layer_index] of this.order.entries()) {
            for (const drawable of this.layer_index_to_drawable_set.get(layer_index)!.entries()) {
                if (TypeGuard.drawable_is_general_renderer(drawable)) {
                    drawable.render();
                } else if (TypeGuard.drawable_is_draw_component(drawable)) {
                    if (this.draw_component_type_to_renderer.has(drawable.type as Type<DrawComponent>)) {
                        const renderer = this.draw_component_type_to_renderer.get(drawable.type as Type<DrawComponent>)!;
                        const entity = this.entity_to_id_map.get(drawable.entity_id)!;

                        if (renderer.is_tracking(entity.id)) {
                            renderer.render(entity);
                        }
                    }
                }
            }
        }
    }
}
